import socket
import threading
import json
import hashlib

class ChatServer:
    def __init__(self, host, port):
        self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.server.bind((host, port))
        self.server.listen(5)
        print(f"Server started on {host}:{port}")
        self.clients = []  # List of connected clients (socket, username)
        self.users = self.load_users()

    def load_users(self):
        try:
            with open('users.json', 'r') as file:
                return json.load(file)
        except FileNotFoundError:
            return {}

    def save_users(self):
        with open('users.json', 'w') as file:
            json.dump(self.users, file)

    def handle_client(self, client_socket, address):
        print(f"New connection from {address}")
        username = None

        while True:
            try:
                request = client_socket.recv(1024).decode()
                if not request:
                    break

                command, *args = request.split(maxsplit=1)

                if command == "REGISTER":
                    username, hashed_password = args[0].split()
                    self.register_user(client_socket, username, hashed_password)
                elif command == "LOGIN":
                    username, hashed_password = args[0].split()
                    if self.login_user(client_socket, username, hashed_password):
                        client_socket.send("LOGIN_SUCCESS".encode())
                        self.clients.append((client_socket, username))
                        self.broadcast(f"{username} has joined the chat.")
                        self.broadcast_user_list()  # Send updated user list
                    else:
                        client_socket.send("LOGIN_FAILED".encode())
                elif command == "MESSAGE":
                    message = args[0]
                    if username:
                        formatted_message = f"{username}: {message}"
                        self.broadcast(formatted_message)

            except ConnectionResetError:
                break

        if username:
            self.broadcast(f"{username} has left the chat.")
            self.clients.remove((client_socket, username))
            self.broadcast_user_list()  # Send updated user list
        client_socket.close()

    def register_user(self, client_socket, username, hashed_password):
        if username in self.users:
            client_socket.send("Username already exists.".encode())
        else:
            self.users[username] = hashed_password
            self.save_users()
            client_socket.send("Registration successful. You can now login.".encode())

    def login_user(self, client_socket, username, hashed_password):
        return username in self.users and self.users[username] == hashed_password

    def broadcast(self, message):
        for client_socket, _ in self.clients:
            try:
                client_socket.send(message.encode())
            except:
                client_socket.close()
                self.clients = [c for c in self.clients if c[0] != client_socket]

    def broadcast_user_list(self):
        users = [username for _, username in self.clients]
        user_list_message = f"USER_LIST {' '.join(users)}"
        for client_socket, _ in self.clients:
            try:
                client_socket.send(user_list_message.encode())
            except:
                client_socket.close()
                self.clients = [c for c in self.clients if c[0] != client_socket]

    def start(self):
        while True:
            client_socket, address = self.server.accept()
            client_handler = threading.Thread(target=self.handle_client, args=(client_socket, address))
            client_handler.start()

if __name__ == "__main__":
    server = ChatServer('0.0.0.0', 5555)
    server.start()